@@ -1,5 +1,5 @@
 /* interpreter -- Tests for the interpreter implementation
- * Copyright (C) 2009-2011  Andrea Bolognani <eof@kiyuko.org>
+ * Copyright (C) 2008-2014  Andrea Bolognani <eof@kiyuko.org>
  * This file is part of Cattle
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,8 +30,12 @@ input_success (CattleInterpreter  *interpreter,
                gpointer            data,
                GError            **error)
 {
-	cattle_interpreter_feed (interpreter,
-	                         "whatever");
+	CattleBuffer *input;
+
+	input = cattle_buffer_new (9);
+	cattle_buffer_set_contents (input, "whatever");
+
+	cattle_interpreter_feed (interpreter, input);
 
 	return TRUE;
 }
@@ -51,8 +55,12 @@ input_utf8 (CattleInterpreter  *interpreter,
             gpointer            data,
             GError            **error)
 {
-	cattle_interpreter_feed (interpreter,
-	                         "\xe2\x84\xa2 (Trademark symbol)");
+	CattleBuffer *input;
+
+	input = cattle_buffer_new (22);
+	cattle_buffer_set_contents (input, "\xe2\x84\xa2 (Trademark symbol)");
+
+	cattle_interpreter_feed (interpreter, input);
 
 	return TRUE;
 }
@@ -63,9 +71,13 @@ input_invalid_utf8 (CattleInterpreter  *interpreter,
                     gpointer            data,
                     GError            **error)
 {
+	CattleBuffer *input;
+
+	input = cattle_buffer_new (3);
+	cattle_buffer_set_contents (input, "\xe2\x28\xa1");
+
 	/* Return some malformed UTF-8 */
-	cattle_interpreter_feed (interpreter,
-	                         "\xe2\x28\xa1");
+	cattle_interpreter_feed (interpreter, input);
 
 	return TRUE;
 }
@@ -78,7 +90,7 @@ input_fail_set_error (CattleInterpreter  *interpreter,
 {
 	g_set_error_literal (error,
 	                     CATTLE_ERROR,
-	                     CATTLE_ERROR_BAD_UTF8,
+	                     CATTLE_ERROR_IO,
 	                     "Spurious error");
 
 	return FALSE;
@@ -101,7 +113,7 @@ input_fail_only_error (CattleInterpreter  *interpreter,
 {
 	g_set_error_literal (error,
 	                     CATTLE_ERROR,
-	                     CATTLE_ERROR_BAD_UTF8,
+	                     CATTLE_ERROR_IO,
 	                     "Spurious error");
 
 	return TRUE;
@@ -110,7 +122,7 @@ input_fail_only_error (CattleInterpreter  *interpreter,
 /* Successful output handler */
 static gboolean
 output_success (CattleInterpreter  *interpreter,
-                gchar               output,
+                gint8               output,
                 gpointer            data,
                 GError            **error)
 {
@@ -120,7 +132,7 @@ output_success (CattleInterpreter  *interpreter,
 /* Succesfull output handler working on a buffer */
 static gboolean
 output_success_buffer (CattleInterpreter  *interpreter,
-                       gchar               output,
+                       gint8               output,
                        gpointer            data,
                        GError            **error)
 {
@@ -129,7 +141,7 @@ output_success_buffer (CattleInterpreter  *interpreter,
 	buffer = (GString*) data;
 
 	g_string_append_c (buffer,
-	                   output);
+	                   (gchar) output);
 
 	return TRUE;
 }
@@ -137,13 +149,13 @@ output_success_buffer (CattleInterpreter  *interpreter,
 /* Unsuccesful output handler that sets the error */
 static gboolean
 output_fail_set_error (CattleInterpreter  *interpreter,
-                       gchar               output,
+                       gint8               output,
                        gpointer            data,
                        GError            **error)
 {
 	g_set_error_literal (error,
 	                     CATTLE_ERROR,
-	                     CATTLE_ERROR_BAD_UTF8,
+	                     CATTLE_ERROR_IO,
 	                     "Spurious error");
 
 	return FALSE;
@@ -152,7 +164,7 @@ output_fail_set_error (CattleInterpreter  *interpreter,
 /* Unsuccesful output handler that doesn't set the error */
 static gboolean
 output_fail_no_set_error (CattleInterpreter  *interpreter,
-                          gchar               output,
+                          gint8               output,
                           gpointer            data,
                           GError            **error)
 {
@@ -162,13 +174,13 @@ output_fail_no_set_error (CattleInterpreter  *interpreter,
 /* Unsuccesful output handler that sets the error but returns TRUE */
 static gboolean
 output_fail_only_error (CattleInterpreter  *interpreter,
-                        gchar               output,
+                        gint8               output,
                         gpointer            data,
                         GError            **error)
 {
 	g_set_error_literal (error,
 	                     CATTLE_ERROR,
-	                     CATTLE_ERROR_BAD_UTF8,
+	                     CATTLE_ERROR_IO,
 	                     "Spurious error");
 
 	return TRUE;
@@ -207,7 +219,7 @@ debug_fail_set_error (CattleInterpreter  *interpreter,
 {
 	g_set_error_literal (error,
 	                     CATTLE_ERROR,
-	                     CATTLE_ERROR_BAD_UTF8,
+	                     CATTLE_ERROR_IO,
 	                     "Spurious error");
 
 	return FALSE;
@@ -230,7 +242,7 @@ debug_fail_only_error (CattleInterpreter  *interpreter,
 {
 	g_set_error_literal (error,
 	                     CATTLE_ERROR,
-	                     CATTLE_ERROR_BAD_UTF8,
+	                     CATTLE_ERROR_IO,
 	                     "Spurious error");
 
 	return TRUE;
@@ -242,41 +254,46 @@ debug_fail_only_error (CattleInterpreter  *interpreter,
 static void
 test_interpreter_handlers (void)
 {
-	CattleInterpreter *interpreter;
+	CattleInterpreter   *interpreter;
 	CattleConfiguration *configuration;
-	CattleProgram *program;
-	GError *error;
-	GString *buffer;
-	gboolean success;
+	CattleProgram       *program;
+	CattleBuffer        *buffer;
+	GError              *error;
+	GString             *output;
+	gboolean             success;
 
 	interpreter = cattle_interpreter_new ();
 
+	buffer = cattle_buffer_new (5);
+	cattle_buffer_set_contents (buffer, ",.,#.");
+
 	configuration = cattle_interpreter_get_configuration (interpreter);
 	cattle_configuration_set_debug_is_enabled (configuration, TRUE);
 	g_object_unref (configuration);
 
 	program = cattle_interpreter_get_program (interpreter);
-	cattle_program_load (program, ",.,#.", NULL);
+	cattle_program_load (program, buffer, NULL);
 	g_object_unref (program);
+	g_object_unref (buffer);
 
-	buffer = g_string_new ("");
+	output = g_string_new ("");
 
 	cattle_interpreter_set_input_handler (interpreter,
 	                                      input_success,
-	                                      buffer);
+	                                      output);
 	cattle_interpreter_set_output_handler (interpreter,
 	                                       output_success_buffer,
-	                                       buffer);
+	                                       output);
 	cattle_interpreter_set_debug_handler (interpreter,
 	                                      debug_success_buffer,
-	                                      buffer);
+	                                      output);
 
 	error = NULL;
 	success = cattle_interpreter_run (interpreter, &error);
 	g_assert (success);
-	g_assert (g_utf8_collate (buffer->str, "w0h") == 0);
+	g_assert (g_utf8_collate (output->str, "w0h") == 0);
 
-	g_string_free (buffer, TRUE);
+	g_string_free (output, TRUE);
 	g_object_unref (interpreter);
 }
 
@@ -291,15 +308,20 @@ static void
 test_interpreter_failed_input (void)
 {
 	CattleInterpreter *interpreter;
-	CattleProgram *program;
-	GError *error;
-	gboolean success;
+	CattleProgram     *program;
+	CattleBuffer      *buffer;
+	GError            *error;
+	gboolean           success;
 
 	interpreter = cattle_interpreter_new ();
 
+	buffer = cattle_buffer_new (1);
+	cattle_buffer_set_contents (buffer, ",");
+
 	program = cattle_interpreter_get_program (interpreter);
-	cattle_program_load (program, ",", NULL);
+	cattle_program_load (program, buffer, NULL);
 	g_object_unref (program);
+	g_object_unref (buffer);
 
 	cattle_interpreter_set_input_handler (interpreter,
 	                                      input_fail_set_error,
@@ -308,7 +330,7 @@ test_interpreter_failed_input (void)
 	error = NULL;
 	success = cattle_interpreter_run (interpreter, &error);
 	g_assert (!success);
-	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_BAD_UTF8));
+	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_IO));
 
 	g_error_free (error);
 
@@ -332,10 +354,9 @@ test_interpreter_failed_input (void)
 	error = NULL;
 	success = cattle_interpreter_run (interpreter, &error);
 	g_assert (!success);
-	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_BAD_UTF8));
+	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_IO));
 
 	g_error_free (error);
-
 	g_object_unref (interpreter);
 }
 
@@ -351,15 +372,18 @@ static void
 test_interpreter_failed_output (void)
 {
 	CattleInterpreter *interpreter;
-	CattleProgram *program;
-	GError *error;
-	gboolean success;
+	CattleProgram     *program;
+	CattleBuffer      *buffer;
+	GError            *error;
+	gboolean           success;
 
 	interpreter = cattle_interpreter_new ();
 
+	buffer = cattle_buffer_new (1);
+	cattle_buffer_set_contents (buffer, ".");
+
 	program = cattle_interpreter_get_program (interpreter);
-	cattle_program_load (program, ".", NULL);
-	g_object_unref (program);
+	cattle_program_load (program, buffer, NULL);
 
 	cattle_interpreter_set_output_handler (interpreter,
 	                                       output_fail_set_error,
@@ -368,7 +392,7 @@ test_interpreter_failed_output (void)
 	error = NULL;
 	success = cattle_interpreter_run (interpreter, &error);
 	g_assert (!success);
-	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_BAD_UTF8));
+	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_IO));
 
 	g_error_free (error);
 
@@ -392,10 +416,12 @@ test_interpreter_failed_output (void)
 	error = NULL;
 	success = cattle_interpreter_run (interpreter, &error);
 	g_assert (!success);
-	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_BAD_UTF8));
+	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_IO));
 
 	g_error_free (error);
 
+	g_object_unref (buffer);
+	g_object_unref (program);
 	g_object_unref (interpreter);
 }
 
@@ -409,21 +435,24 @@ test_interpreter_failed_output (void)
 static void
 test_interpreter_failed_debug (void)
 {
-	CattleInterpreter *interpreter;
+	CattleInterpreter   *interpreter;
 	CattleConfiguration *configuration;
-	CattleProgram *program;
-	GError *error;
-	gboolean success;
+	CattleProgram       *program;
+	CattleBuffer        *buffer;
+	GError              *error;
+	gboolean             success;
 
 	interpreter = cattle_interpreter_new ();
 
+	buffer = cattle_buffer_new (1);
+	cattle_buffer_set_contents (buffer, "#");
+
 	configuration = cattle_interpreter_get_configuration (interpreter);
 	cattle_configuration_set_debug_is_enabled (configuration, TRUE);
 	g_object_unref (configuration);
 
 	program = cattle_interpreter_get_program (interpreter);
-	cattle_program_load (program, "#", NULL);
-	g_object_unref (program);
+	cattle_program_load (program, buffer, NULL);
 
 	cattle_interpreter_set_debug_handler (interpreter,
 	                                      debug_fail_set_error,
@@ -432,7 +461,7 @@ test_interpreter_failed_debug (void)
 	error = NULL;
 	success = cattle_interpreter_run (interpreter, &error);
 	g_assert (!success);
-	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_BAD_UTF8));
+	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_IO));
 
 	g_error_free (error);
 
@@ -456,10 +485,12 @@ test_interpreter_failed_debug (void)
 	error = NULL;
 	success = cattle_interpreter_run (interpreter, &error);
 	g_assert (!success);
-	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_BAD_UTF8));
+	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_IO));
 
 	g_error_free (error);
 
+	g_object_unref (buffer);
+	g_object_unref (program);
 	g_object_unref (interpreter);
 }
 
@@ -473,15 +504,18 @@ static void
 test_interpreter_input_no_feed (void)
 {
 	CattleInterpreter *interpreter;
-	CattleProgram *program;
-	GError *error;
-	gboolean success;
+	CattleProgram     *program;
+	CattleBuffer      *buffer;
+	GError            *error;
+	gboolean           success;
 
 	interpreter = cattle_interpreter_new ();
 
+	buffer = cattle_buffer_new (1);
+	cattle_buffer_set_contents (buffer, ",");
+
 	program = cattle_interpreter_get_program (interpreter);
-	cattle_program_load (program, ",", NULL);
-	g_object_unref (program);
+	cattle_program_load (program, buffer, NULL);
 
 	cattle_interpreter_set_input_handler (interpreter,
 	                                      input_no_feed,
@@ -493,6 +527,8 @@ test_interpreter_input_no_feed (void)
 	g_assert (success);
 	g_assert (error == NULL);
 
+	g_object_unref (buffer);
+	g_object_unref (program);
 	g_object_unref (interpreter);
 }
 
@@ -505,15 +541,18 @@ static void
 test_interpreter_unicode_input (void)
 {
 	CattleInterpreter *interpreter;
-	CattleProgram *program;
-	GError *error;
-	gboolean success;
+	CattleProgram     *program;
+	CattleBuffer      *buffer;
+	GError            *error;
+	gboolean           success;
 
 	interpreter = cattle_interpreter_new ();
 
+	buffer = cattle_buffer_new (1);
+	cattle_buffer_set_contents (buffer, ",");
+
 	program = cattle_interpreter_get_program (interpreter);
-	cattle_program_load (program, ",", NULL);
-	g_object_unref (program);
+	cattle_program_load (program, buffer, NULL);
 
 	cattle_interpreter_set_input_handler (interpreter,
 	                                      input_utf8,
@@ -521,11 +560,12 @@ test_interpreter_unicode_input (void)
 
 	error = NULL;
 	success = cattle_interpreter_run (interpreter, &error);
-	g_assert (!success);
-	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_INPUT_OUT_OF_RANGE));
 
-	g_error_free (error);
+	g_assert (success);
+	g_assert (error == NULL);
 
+	g_object_unref (buffer);
+	g_object_unref (program);
 	g_object_unref (interpreter);
 }
 
@@ -538,15 +578,18 @@ static void
 test_interpreter_invalid_input (void)
 {
 	CattleInterpreter *interpreter;
-	CattleProgram *program;
-	GError *error;
-	gboolean success;
+	CattleProgram     *program;
+	CattleBuffer      *buffer;
+	GError            *error;
+	gboolean           success;
 
 	interpreter = cattle_interpreter_new ();
 
+	buffer = cattle_buffer_new (1);
+	cattle_buffer_set_contents (buffer, ",");
+
 	program = cattle_interpreter_get_program (interpreter);
-	cattle_program_load (program, ",", NULL);
-	g_object_unref (program);
+	cattle_program_load (program, buffer, NULL);
 
 	cattle_interpreter_set_input_handler (interpreter,
 	                                      input_invalid_utf8,
@@ -554,11 +597,11 @@ test_interpreter_invalid_input (void)
 
 	error = NULL;
 	success = cattle_interpreter_run (interpreter, &error);
-	g_assert (!success);
-	g_assert (g_error_matches (error, CATTLE_ERROR, CATTLE_ERROR_BAD_UTF8));
-
-	g_error_free (error);
+	g_assert (success);
+	g_assert (error == NULL);
 
+	g_object_unref (buffer);
+	g_object_unref (program);
 	g_object_unref (interpreter);
 }
 
@@ -574,12 +617,12 @@ static void
 test_interpreter_unbalanced_brackets (void)
 {
 	CattleInterpreter *interpreter;
-	CattleProgram *program;
+	CattleProgram     *program;
 	CattleInstruction *instructions;
 	CattleInstruction *next;
-	CattleTape *tape;
-	GError *error;
-	gboolean success;
+	CattleTape        *tape;
+	GError            *error;
+	gboolean           success;
 
 	interpreter = cattle_interpreter_new ();
 
@@ -629,9 +672,13 @@ test_interpreter_unbalanced_brackets (void)
 }
 
 gint
-main (gint argc, gchar **argv)
+main (gint    argc,
+      gchar **argv)
 {
+#if !GLIB_CHECK_VERSION(2, 36, 0)
 	g_type_init ();
+#endif
+
 	g_test_init (&argc, &argv, NULL);
 
 	g_test_add_func ("/interpreter/handlers",
